@@ -20,7 +20,7 @@ module ConfigParser.Parsertest(tests) where
 import Test.HUnit
 import MissingH.ConfigParser.Parser
 import MissingH.ConfigParser.Types
-import Testutil
+import MissingH.HUnit
 import Control.Exception
 
 test_basic =
@@ -86,4 +86,4 @@ test_extensionlines =
 tests = TestList [TestLabel "test_basic" (TestList test_basic),
                   TestLabel "test_asserts" (TestList test_asserts),
                   TestLabel "test_extensionlines" (TestCase test_extensionlines)
-                 ]
\ No newline at end of file
+                 ]
