@@ -19,8 +19,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 module Strtest(tests) where
 import Test.HUnit
 import MissingH.Str
-import Testutil
+import MissingH.HUnit
 import Text.Regex
+import Data.Char
 
 test_lstrip =
     mapassertEqual "lstrip" lstrip
@@ -51,6 +52,16 @@ test_strip =
                     ("\nbas", "bas"),
                     ("abc def", "abc def")]
 
+test_splitWs =
+    let f exp inp = TestCase $ exp @=? splitWs inp
+        in [
+            f [] "    ",
+            f [] "",
+            f ["asdf"] " asdf\n",
+            f ["one", "two", "three"] "  one\ntwo \tthree \n"
+           ]
+
+
 test_splitRe =
     let f re inp exp = TestCase $ exp @=? splitRe (mkRegex re) inp
         in [
@@ -73,11 +84,24 @@ test_subRe =
            ,f "foo" "Test foo bar" "x\\\\x" "Test x\\x bar"
            ]
 
+test_escapeRe =
+    map (\i -> TestLabel (show $ chr i) $ TestCase $ assertEqual [chr i] (Just []) 
+                (matchRegex (mkRegex $ escapeRe $ [chr i]) [chr i]))
+             [1..255]
+    ++
+    [TestCase $ assertEqual "big string" 
+                     (Just ([], teststr, [], []))
+                     (matchRegexAll (mkRegex $ escapeRe teststr) teststr)
+    ]
+    where teststr = map chr [1..255]
+
 tests = TestList [TestLabel "lstrip" (TestList test_lstrip),
                   TestLabel "rstrip" $ TestList test_rstrip,
                   TestLabel "strip" $ TestList test_strip,
+                  TestLabel "splitWs" $ TestList test_splitWs,
                   TestLabel "subRe" $ TestList test_subRe,
-                  TestLabel "splitRe" $ TestList test_splitRe
+                  TestLabel "splitRe" $ TestList test_splitRe,
+                  TestLabel "escapeRe" $ TestList test_escapeRe
                   ]
 
 
